## Copyright (C) 2010 Fotios Kasolis
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## PUBLISH takes a script or function file executes it and creates
## a tex and a pdf document with the source code nicely formated ,
## execution results, and a (at most) figure.
##
## fileName      : <string>  : name of script or function file with
##                             or without extension.
## evalCode(true): <boolean> : execute and include results (or not).
## funcArgs("")  : <string>  : string that contains the arguments
##                             to be used in the form "arg1,arg2,
##                             arg3,..". If this is non-empty the
##                             given file is assumed to be func.
## imagFile("")  : <string>  : name of (strictly) pdf image with or
##                             without extension.
##
## Example 1: Script file without figure
##    > publish(myscript)
##
## Example 2: Script file without execution results
##    > publish(myscript,false)
##
## Example 3: Function file
##    > publish(myfunc,true,"1,2,'test'");
##
## Example 4: Function file without execution results
##    > publish(myfunc,false);
##
## Example 5: Script file with execution results and figure
##    > publish(myscript,true,[],"figfile")
##
## WARNING! Line 150 remove files using SYSTEM()

## Author: Fotios Kasolis <fotios@Fotios-Kasoliss-MacBook.local>
## Created: 2010-06-24

function publish (fileName, evalCode=true, funcArgs="", imagFile="")

if (nargin < 1) || (nargin > 4)
  error("Invalid call to publish.");
endif

if !strcmp(fileName(end-1:end), ".m")
  iFile = [fileName, ".m"];
  oFile = [fileName, ".tex"];
else
  iFile = fileName;
  oFile = [fileName(1:end-1), "tex"];
endif

if (exist(iFile) != 2) && (exist(iFile) != 3)
  error(["File ", iFile, " does not exist."]);
endif

if !isbool(evalCode)
  error("evalCode accepts logical value.");
  endif

if !ischar(imagFile)
  error("imagFile accepts string value.");
endif

latexPreamble = ["\\documentclass[a4paper,12pt]{article}\n",...
                 "\\usepackage{listings}\n",...
                 "\\usepackage{graphicx}\n",...
                 "\\usepackage{color}\n"...
                 "\\usepackage[T1]{fontenc}\n"...
                 "\\definecolor{lightgray}{rgb}{0.9,0.9,0.9}\n"];
listingOption = ["\\lstset{\n",...
                 "language = Octave,\n",...
                 "basicstyle =\\footnotesize,\n",...
                 "numbers = left,\n",...
                 "numberstyle = \\footnotesize,\n",...
                 "backgroundcolor=\\color{lightgray},\n",...
                 "frame=single,\n",...
                 "tabsize=2,\n",...
                 "breaklines=true}\n"];

section1Title  = ["\\section*{Source code: \\texttt{" iFile "}}\n"];
codeFile       = ["\\lstinputlisting{" iFile "}\n"];

if evalCode
  section2Title = ["\\section*{Execution results}\n"];
  if isempty(funcArgs)
    system(["octave -q --eval ", iFile(1:end-2), " > ",...
            "publishTmpScript.txt"]);
    octCommand = ["\\begin{lstlisting}[numbers=none]\n",...
                  "octave> ", iFile(1:end-2), "\n",...
                  "\\end{lstlisting}\n"];
    execResult = ["\\lstinputlisting[frame=none, numbers=none,...
                        backgroundcolor=\\color{white}]{publishTmpScript.txt}\n"];
  else
    funcCall = [iFile(1:end-2), "(", funcArgs ,")"];
    system(["octave -q --eval \"", funcCall, "\" > ",...
              "publishTmpFunc.txt"]);
    octCommand = ["\\begin{lstlisting}[numbers=none]\n",...
                  "octave> ", funcCall, "\n",...
                  "\\end{lstlisting}\n"];
    execResult = ["\\lstinputlisting[frame=none, numbers=none,...
                        backgroundcolor=\\color{white}]{publishTmpFunc.txt}\n"];
  endif
else
  section2Title = "";
  octCommand    = "";
  execResult    = "";
endif

if !isempty(imagFile)
  dispFig = ["\\begin{figure}[htb!]\n",...
             "\\centering\n",...
             "\\includegraphics[scale=0.6]{", imagFile, "}\n",...
             "\\caption{Graphics generated by executing \\texttt{", ...
                      iFile, "}}\n",...
             "\\label{fig:", iFile(1,end-2), "}\n",...
             "\\end{figure}\n"];
else
  dispFig = "";
endif

finalDocument = [latexPreamble,...
                 listingOption,...
                 "\\begin{document}\n",...
                 section1Title,...
                 codeFile,...
                 section2Title,...
                 octCommand,...
                 execResult,...
                 dispFig,...
                 "\\end{document}"];

fid = fopen(oFile, "wt");
fprintf(fid, finalDocument);
fclose(fid);

system(["pdflatex ", oFile]);
system(["rm *.aux *.log publishTmp*"]);
endfunction
